/*

CommandSequence

K			LightWave 6.0
R|[lg	f[
wb_[		lwcmdseq.h

LWMODCOMMAND_CLASS
LWMODCOMMAND_VERSION

#include <lwcmdseq.h>
*/

#include "main.h"


static LWID	g_vmapType[] =		{ LWVMAP_WGHT,	LWVMAP_MORF,	LWVMAP_SPOT	};
static int	g_vmapDimension[] = { 1,			3,				3			};

void readConfig(MyData *data);
void writeConfig(MyData *data);
EDError PointScanFunc(void *scanData, const EDPointInfo *info);


//ANeBx[V֐
XCALL_(int)Activate(long version, GlobalFunc *global, LWModCommand *local, void *serverData)
{
	LWMessageFuncs	*msgf = (LWMessageFuncs*)global( LWMESSAGEFUNCS_GLOBAL, GFUSE_TRANSIENT );
	MyData			data;

	if( LWMODCOMMAND_VERSION != version )
		return AFUNC_BADVERSION;
	if( !msgf )
		return AFUNC_BADGLOBAL;

	data.global = global;

	//ݒǂݍ
	readConfig(&data);
	data.mapType = 0;

	//SEGCg}bvXg쐬
	listupAllWMap(&data);
	if( 0 == data.allwlist.size() ){
		msgf->error("EFCg}bv܂B", NULL);
		return AFUNC_OK;
	}

	//EGCg}bvXg쐬
	listupEditWMap(&data);

	//pl\
	if( 0 == Panel(&data, global) ){
		return AFUNC_OK;
	}

	//ݒۑ
	writeConfig(&data);

	data.edit = local->editBegin(0, 0, OPSEL_GLOBAL);
	//|CgXL
	data.edit->pointScan(data.edit->state, PointScanFunc, &data, OPLYR_PRIMARY);

	data.edit->done(data.edit->state, EDERR_NONE, 0);
	return AFUNC_OK;
}




//|CgXL֐
EDError PointScanFunc(void *scanData, const EDPointInfo *info)
{
	MyData		*data = (MyData*)scanData;
	MeshEditOp	*edit = data->edit;
	int			result;
	LWFVector	val;

	for(int i=0; i<data->wlist.size(); i++){
		data->wlist[i].mapID = (LWID)edit->pointVSet(edit->state, (void*)data->wlist[i].mapID, g_vmapType[data->mapType], data->wlist[i].src.c_str());
		result = edit->pointVGet(edit->state, info->pnt, val);
		if( 0 != result && data->wlist[i].active ){
			if( 0.0 < (info->position[0] * data->operation) ){
				edit->pntVMap(edit->state, info->pnt, g_vmapType[data->mapType], data->wlist[i].src.c_str(), g_vmapDimension[data->mapType], NULL);
				edit->pntVMap(edit->state, info->pnt, g_vmapType[data->mapType], data->wlist[i].dist.c_str(), g_vmapDimension[data->mapType], NULL);
				edit->pntVMap(edit->state, info->pnt, g_vmapType[data->mapType], data->wlist[i].dist.c_str(), g_vmapDimension[data->mapType], val);
			}else if( 0.0 == info->position[0] ){
				edit->pntVMap(edit->state, info->pnt, g_vmapType[data->mapType], data->wlist[i].dist.c_str(), g_vmapDimension[data->mapType], val);
			}
		}
	}

	return EDERR_NONE;
}





void readConfig(MyData *data)
{
	LWDirInfoFunc	*dirinfo = (LWDirInfoFunc*)data->global( LWDIRINFOFUNC_GLOBAL, GFUSE_TRANSIENT );
	char			filename[MAX_PATH], buffer[MAX_PATH];
	FILE			*fp;
	SettingData		setting;

	if( !dirinfo ){
		goto CFGREADFAILED;
	}

	sprintf(filename, "%s\\mirrorweight.cfg.txt", dirinfo(LWFTYPE_SETTING));
	fp = fopen(filename, "rt");
	if( !fp ){
		goto CFGREADFAILED;
	}
	if( 0 >= fscanf(fp, "%s", buffer) ){
		fclose(fp);
		goto CFGREADFAILED;
	}
	data->srcKey = buffer;
	fscanf(fp, "%s", buffer);
	data->distKey = buffer;
	fscanf(fp, "%d", &data->operation);
	fscanf(fp, "%d", &data->replace);
	
	while( 0 < fscanf(fp, "%s", buffer) ){
		setting.name = buffer;
		fscanf(fp, "%s", buffer);
		setting.srcKey = buffer;
		fscanf(fp, "%s", buffer);
		setting.distKey = buffer;
		fscanf(fp, "%d", &setting.operation);
		fscanf(fp, "%d", &setting.replace);
		data->settings.push_back(setting);
	}

	fclose(fp);
	return;

CFGREADFAILED:
	data->distKey = "Right";
	data->srcKey = "Left";
	data->operation = -1;
	data->replace = 0;
	return ;
}

void writeConfig(MyData *data)
{
	LWDirInfoFunc	*dirinfo = (LWDirInfoFunc*)data->global( LWDIRINFOFUNC_GLOBAL, GFUSE_TRANSIENT );
	char			filename[MAX_PATH];
	FILE			*fp;

	if( !dirinfo ){
		return;
	}

	sprintf(filename, "%s\\mirrorweight.cfg.txt", dirinfo(LWFTYPE_SETTING));
	fp = fopen(filename, "wt");
	if( !fp ){
		return;
	}
	fprintf(fp, "%s\n", data->srcKey.c_str());
	fprintf(fp, "%s\n", data->distKey.c_str());
	fprintf(fp, "%d\n", data->operation);
	fprintf(fp, "%d\n", data->replace);

	std::vector<SettingData>::iterator	itr, itr_end;
	itr = data->settings.begin();
	itr_end = data->settings.end();

	for(; itr!=itr_end; itr++){
		fprintf(fp, "%s\n", itr->name.c_str());
		fprintf(fp, "%s\n", itr->srcKey.c_str());
		fprintf(fp, "%s\n", itr->distKey.c_str());
		fprintf(fp, "%d\n", itr->operation);
		fprintf(fp, "%d\n", itr->replace);
	}

	fclose(fp);
}





void listupAllWMap(MyData *data)
{
	LWObjectFuncs	*objfunc = (LWObjectFuncs*)data->global( LWOBJECTFUNCS_GLOBAL, GFUSE_TRANSIENT );
	WeightListData	listdata;

	data->allwlist.clear();

	if( !objfunc ) return;

	int num = objfunc->numVMaps(g_vmapType[data->mapType]);
	for(int i=0; i<num; i++){
		listdata.src = objfunc->vmapName(g_vmapType[data->mapType], i);
		data->allwlist.push_back(listdata);
	}
}

void listupEditWMap(MyData *data)
{
	WeightListData	listdata;
	char			buffer[MAX_PATH], *p;
	int				i;
	const char		*srckey, *distkey, *vmapname;
	int				len;

	data->wlist.clear();

	srckey = data->srcKey.c_str();
	distkey = data->distKey.c_str();
	len = strlen(srckey);
	listdata.active = 1;
	listdata.mapID = NULL;

	if( 0 == data->replace ){
		for(i=0; i<data->allwlist.size(); i++){
			vmapname = data->allwlist[i].src.c_str();
			if( 0 == strncmp(srckey, vmapname, len) ){
				listdata.src = vmapname;
				sprintf(buffer, "%s%s", distkey, (vmapname+len));
				listdata.dist = buffer;
				data->wlist.push_back(listdata);
			}
		}
	}else{
		for(i=0; i<data->allwlist.size(); i++){
			vmapname = data->allwlist[i].src.c_str();
			strcpy(buffer, vmapname);
			p = strstr(buffer, srckey);
			if( p ){
				*p = '\0';
				strcat(buffer, distkey);
				listdata.src = vmapname;
				listdata.dist = buffer;
				data->wlist.push_back(listdata);
			}
		}
	}
}

